
# q-expansion of newform 1407.2.a.j, downloaded from the LMFDB on 05 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1407
weight = 2
poly_data = [-1, -3, 11, 4, -7, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[1, 1, -4, -1, 1, 0], 2], [[0, -5, -3, 6, 1, -1], 2], [[5, 3, -6, -1, 1, 0], 2], [[-2, 9, 3, -6, -1, 1], 2], [[-4, 15, 5, -8, -1, 1], 2]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0, 1, 0, 0], [-1, 0, 0, 0, 0, 0], [1, 0, -1, 0, 1, 0], [1, 0, 0, 0, 0, 0], [1, 0, 1, -1, 1, -1], [3, 0, 0, 0, 1, -1], [2, 0, 0, 0, -1, -1], [-1, -2, -1, 0, 0, -1], [-1, -2, -1, 0, 0, 1], [1, 0, -1, 0, -2, -1], [0, 2, 1, 0, 0, -1], [-1, 0, 2, 0, 0, -2], [6, -2, 0, 0, 0, 0], [-3, 2, 1, -2, 1, 0], [2, -2, 2, 3, -1, 0], [2, 4, 0, -2, 0, -4], [2, 2, 4, 1, 1, 0], [-2, 0, 3, 2, -2, 1], [-1, 0, 0, 0, 0, 0], [2, 2, 2, 0, 0, 2], [1, 2, -1, 2, 0, 5], [-3, -2, -3, 0, -1, 2], [0, 0, 2, -4, -4, 0], [0, -2, -2, -1, -3, 2], [5, -2, -2, 0, 1, 5], [3, 2, 3, 0, 1, 4], [-6, 0, -4, 2, -2, 2], [5, -4, -3, 0, 0, 3], [-2, 4, 4, 0, 2, 0], [3, 2, -1, -4, 3, 0], [-3, 0, 2, -2, -2, -2], [-2, 2, -6, -3, 2, -1], [8, -2, 2, 3, -2, -1], [1, 2, 4, -6, -1, -1], [-1, 4, -5, -2, 2, 1], [-4, 0, 0, 2, -5, 3], [7, -2, -3, 0, 6, -3], [5, -2, 2, -2, 4, -4], [2, -2, 8, -3, -2, -5], [4, 2, 4, -5, 1, 2], [-4, -2, 0, 3, 2, 1], [-1, -2, 1, -6, 0, 1], [8, -2, 4, -1, -4, -1], [1, -6, 4, 0, -2, -6], [-1, -4, -3, 1, -5, 3], [3, -2, 3, 6, -4, -1], [-9, -2, -4, -4, -1, 1], [1, -2, -1, 2, -2, 7], [4, 2, 0, -1, 5, 4], [3, -6, -9, 6, -1, 0], [-2, -2, 4, -1, -2, 5], [-8, 4, 6, 0, 0, 0], [7, -2, 5, 6, -2, 3], [-3, 10, -1, 2, 1, 2], [4, 6, 2, 3, 1, -2], [-4, 6, -10, 2, 2, 6], [6, 6, 2, 5, 0, -1], [3, 6, -4, -2, 1, 1], [3, -6, 0, 2, -3, 5], [8, -10, 0, 5, -2, -3], [-3, -10, -1, 2, -4, -3], [-2, -2, -4, -3, 4, 3], [-5, 6, -3, -10, 0, -1], [-12, -2, 2, -2, -4, -4], [2, 4, 0, -12, 0, -4], [2, 2, 4, -8, 2, 2], [5, -2, 1, 4, 3, 6], [5, -2, -3, -4, -7, -4], [6, 2, -2, 1, 4, -5], [0, 0, 0, -10, 6, -2], [5, -6, -5, 4, -1, -2], [9, 0, 5, 8, 2, 5], [-2, 12, 3, -4, 0, 1], [-11, -2, -5, 8, 3, 10], [-3, -10, -13, 8, 1, 2], [11, -4, 3, 4, -1, -2], [6, 0, 4, -4, 8, -4], [-7, 2, 7, 2, -4, 5], [-2, -4, -12, 6, -4, 8], [6, 4, 7, 2, 0, 3], [2, 2, 8, 5, 3, -6], [1, -2, 0, 8, 0, 4], [-9, -4, 1, 10, -4, 1], [8, -2, 9, -6, -8, -7], [1, 6, -7, -2, 10, 1], [9, 10, 5, -6, 3, 8], [3, -4, -9, 8, 4, -7], [8, 0, -2, 6, -3, 7], [10, 0, -4, 0, 5, -3], [1, -2, -9, 6, -1, 4], [-8, -2, -6, 5, 0, 3], [4, -2, -8, 9, -7, 6], [-9, -6, -1, -8, 1, 0], [3, -8, 9, 8, -8, -1], [-14, 0, 2, -2, 0, -4], [-5, 4, 1, 6, 3, 10], [4, -8, -6, 8, -1, 1], [5, -2, -5, -4, -7, 4], [9, 2, 17, -6, 4, -11], [6, 4, 14, -6, -2, -4], [3, 2, -11, 2, -1, 6], [2, 0, -6, 12, -4, 2], [5, -2, 3, -4, -5, -10], [-3, 0, -5, 4, 4, 9], [-15, -6, 4, -6, -6, 8], [29, -2, 1, -2, 3, -2], [-1, -2, -5, -6, 5, -10], [-2, 0, 10, -2, -4, -4], [5, -12, -7, 1, -3, 1], [19, 6, 5, -2, -4, 1], [-4, 0, -14, 2, -4, 10], [-15, 4, 4, -4, -10, 6], [5, 2, -1, -12, 0, 5], [-4, -4, 0, 14, 0, 6], [6, -8, 0, -8, 4, -2], [-14, -2, -2, -1, -2, 5], [-2, 12, 0, 8, 2, 4], [-5, -4, 5, -2, -11, 8], [-1, 4, 7, -9, 3, 1], [-29, -4, -9, 12, 4, 9], [3, -10, -9, -6, 9, 2], [6, 0, 6, 4, -10, 14], [16, 4, -4, -10, 2, 4], [3, 2, -5, 10, 11, -2], [-11, 2, 11, -2, -14, 9], [2, 12, -8, -2, 4, 4], [-25, 0, 2, -6, -5, -3], [-2, -4, -16, 2, 5, 3], [-5, -8, -2, -6, -1, 1], [16, 4, 3, 6, -8, 3], [-21, -6, -5, 0, -3, 6], [0, 4, 2, 6, 4, 0], [-13, 0, -4, -6, 4, -2], [6, -4, 4, 8, 0, 4], [0, -6, -4, 3, -7, -2], [-9, 6, 3, 6, -5, 12], [0, 10, -4, 9, 7, 2], [-11, 0, 12, 2, -5, -5], [-4, 0, -6, 6, 6, 0], [1, 6, 9, -12, -5, -6], [5, 6, -5, -18, 1, -2], [-3, -4, -3, 0, 10, -7], [-4, 12, 4, 2, -1, -1], [-11, 6, -7, -8, 10, 1], [-3, 10, -3, -8, 12, -11], [-8, 4, -10, -10, 7, -1], [15, -2, 3, 2, -14, -1], [18, -6, 14, -8, 4, -10], [-10, -4, 0, 8, -6, 10], [7, 6, 5, 4, -12, -5], [-15, -6, -2, 4, -2, 0], [-18, -6, 6, 13, -9, 4], [-2, 12, 2, 14, -6, -4], [-2, -2, -4, 3, 5, -6], [-11, -8, -2, 14, 0, -4], [5, -6, 3, -4, -2, -13], [-12, -12, -18, 12, 6, 0], [10, -4, -8, 4, -2, 10], [10, 0, 15, -2, 8, -9], [2, -6, -2, -4, -6, 4], [-11, 2, -3, -12, 6, -7], [1, 4, 19, 2, -4, -9], [-3, -4, 10, 12, 3, -5], [2, 0, -16, -2, 5, -5], [21, 2, 5, 2, 4, 13], [18, 0, 0, 0, -2, -2], [-19, 6, -7, -10, -7, 6], [0, 8, -10, 8, 10, 2], [-13, -10, 6, 8, -10, -4], [-10, 0, -2, 12, -2, 12], [1, 18, 5, -18, 4, -7], [-19, 6, 1, -4, -8, -7], [-7, 18, 11, -4, 4, 9], [24, -12, 4, 6, 0, -6], [5, -10, 0, 2, -9, -5], [-17, 12, 9, -6, 1, 8], [-5, 10, 3, -6, 11, -10], [-21, -4, 5, 12, -4, -1], [2, 8, -3, 0, 12, 11], [-7, 2, 3, -4, -10, -7], [-7, 6, 8, -4, 9, -9], [4, 2, 4, -3, -8, 15], [-8, 0, -10, 10, -5, -7], [12, -14, -12, 1, 6, -1], [19, -6, 11, -10, 7, -14], [-10, 0, -4, -10, 0, -4], [5, 6, 1, -4, -1, 6], [-10, 16, -3, 8, 0, 3], [-1, -2, -9, 10, -3, -4], [8, 0, -4, 4, 6, -2], [10, -4, -16, 0, 0, 10], [-11, 2, -7, 2, -8, 17], [8, -4, -12, 6, 8, -6], [-19, -2, -13, -14, 9, -6], [20, -16, -14, 4, 2, 6], [-2, -2, -12, 3, 0, 5], [5, 8, 2, -28, 8, -2], [-15, 6, 3, 12, -1, 4], [-3, 10, 7, -6, -5, 0], [-10, -6, 6, 17, -2, -9], [4, 6, 12, -2, -12, 2], [-1, -10, -17, -4, 14, -7], [0, -8, 6, 0, 10, 12], [-19, -4, 4, 16, -3, -1], [-3, 8, 1, -14, 11, -18], [10, -2, 4, 1, -7, 2], [2, -20, -2, 16, -10, 6], [3, -14, -3, 6, -2, 15], [-11, -2, 19, 8, -10, -9], [-20, 8, 3, 0, -12, -11], [-4, 20, 2, 0, -6, -8], [7, 8, 1, 8, 7, -16], [-11, -6, -15, -2, 1, -4], [3, 8, 11, -2, -11, 14], [1, -6, 13, 6, 5, 8], [19, -6, 12, 0, 9, -1], [-1, -2, -6, -10, -4, 12], [4, -4, 2, -6, 6, -4], [4, 10, 10, -3, -2, -9], [-3, 6, 1, -12, 10, -3], [-13, 10, -11, -12, -9, -8], [-1, 6, 1, -4, 14, -9], [-11, -2, -3, -4, -15, -8], [4, 0, -4, 14, -5, -9], [-2, 8, 0, 0, 2, -2], [-9, 22, 2, -4, -2, -4], [10, 20, -4, -6, 8, -8], [9, 12, 5, -11, 1, -7], [-5, 18, 5, 4, 2, -9], [-7, -8, -5, 17, -5, 19], [-8, -8, -22, 6, 10, -4], [9, 8, 4, -18, -3, 9], [6, 4, -2, -2, 5, 15], [17, 4, 13, -19, -3, -5], [19, 2, 9, -2, -16, -3], [22, -2, 6, -2, -6, -2], [-9, 14, 3, -10, 14, 1], [8, -6, 10, -6, 4, 4], [-23, -8, -5, 8, -2, -9], [6, -10, -16, -5, 14, 1], [21, -14, 5, -12, 6, -19], [-14, 12, 7, 0, -16, -11], [1, -26, -8, 2, -1, -1], [8, 8, 12, 2, 2, 2], [-41, 0, 1, -4, -1, -4], [10, -18, -14, -8, -2, 12], [3, -6, -11, 4, 12, -11], [26, 4, -8, -12, 10, -6], [-28, 8, -8, -8, -14, 8], [-11, 8, -9, 11, -3, 3], [4, 4, -6, 6, 11, 1], [-4, -6, -6, -7, -6, -5], [2, 2, -14, 1, -8, -1], [1, -18, -17, -6, 8, 1], [-15, -4, 5, -6, -11, 8], [-11, 0, -9, -9, -9, 13], [-9, -10, 15, -4, -17, -6], [6, 16, -10, -8, 10, 0], [-12, 0, 10, 6, 2, 16], [43, -14, -7, 6, -12, 11], [10, 12, -2, -10, -1, 9], [22, -24, -4, 16, -2, 2], [-11, -6, -15, 22, 1, -4], [12, -8, -5, -6, -8, 3], [12, 2, -16, -9, 2, 1], [-13, -2, -5, -8, 0, 5], [8, 0, 10, -2, 4, 10], [11, -18, 1, 6, 3, -10], [-1, 6, 11, -12, 1, -20], [17, -4, -3, 15, -19, 13], [-10, 6, 15, -6, -14, -7], [47, 2, 1, 2, 7, -12], [1, -20, -14, 0, 13, 13], [-18, 4, -6, 12, -10, 12], [13, 6, 13, -6, 8, -9], [5, 12, 4, -22, -3, -11], [34, 0, 12, 8, -4, 10], [-11, 4, -6, 2, -3, 3], [17, -8, 14, 14, -7, 11], [8, 0, -2, 18, -20, 12], [0, -20, -2, -10, 3, -1], [11, -12, 6, -16, 9, -11], [-10, 30, 2, -14, 8, 0], [25, 4, 16, -8, -3, -17], [-13, -4, -12, -2, -13, -17], [28, -6, 10, -2, -18, 2], [-1, -2, 12, 4, -16, 24], [-19, 12, 5, -10, -10, -1], [8, 8, -12, -8, 10, 10], [2, -12, -14, -4, -8, 6], [-1, -4, -9, -6, -4, -5], [21, 6, 9, 18, -13, 14], [-28, 28, 6, 2, -6, 6], [10, 4, 4, 16, -2, 20], [-21, 2, 11, -8, -19, 8], [2, -18, -12, 9, 0, -5], [8, 12, 20, -20, 2, -14], [-5, 6, -1, 12, -5, -10], [-20, 18, 14, -1, 9, 4], [-8, 4, 4, -8, 4, -4], [-9, -18, -29, 2, 3, 0], [9, -4, -11, -4, 10, -7], [3, 0, 2, 20, 5, 5], [16, -6, 12, -10, 2, -16], [-38, -4, -2, -18, 3, 1], [17, 6, 11, -28, 9, -8], [31, -6, 1, 10, -18, -1], [1, -10, 5, 8, 10, 5], [-7, 18, 13, -14, -4, 9], [-33, 6, 5, 6, 9, -14], [22, 8, -2, 16, 4, 6], [-41, 14, 15, 0, -8, -7], [-27, 4, -5, 10, 12, 15], [8, 20, -8, -2, 9, 9], [-5, -2, -17, -6, 20, -13], [5, -14, -12, 24, -9, -3], [24, -10, 8, 13, -11, -2], [3, 20, 25, -15, -11, -1], [-13, 6, -3, 14, 2, -13], [30, -6, 2, -3, 2, -13], [3, 10, 3, 0, 4, 7], [23, -4, 2, -8, 20, -6], [-2, 10, 26, -15, 9, -2], [8, -12, 6, 10, -2, -8], [-11, 32, 25, -14, -2, -1], [-12, 4, -16, 4, 18, 0], [15, 12, 10, 10, -8, 16], [-18, 4, 6, 0, -10, 6], [10, -12, -12, -16, 20, -10], [27, 0, 17, -23, -1, -21], [-29, 14, 12, -12, -8, 12], [-7, -2, 13, -10, -20, 13], [-11, -10, -37, 18, 5, 2], [-2, 16, 8, 10, -8, -14], [-1, -28, -2, -8, -3, -15], [-30, 0, -20, 6, 7, 5], [28, 4, 4, 8, 2, -2], [-4, -4, 10, -8, 17, -1], [-15, 0, -16, -24, 5, -1], [29, 10, 17, -8, 0, -5], [1, -22, -2, 2, -5, 5], [42, 14, 0, -10, 2, -16], [-3, -12, -11, -6, 17, 8], [-15, 2, -16, -18, 12, 2], [19, 16, -3, 20, -6, 5], [-3, 10, -11, -22, 11, -16], [38, 6, 9, 10, 6, 11], [-6, 8, 28, -6, 4, 0], [2, 10, 26, -15, 2, -9], [-14, -12, 2, 20, -4, 2], [-12, -8, 10, 6, -4, 0], [22, -4, -6, 14, 8, -8], [9, -26, -33, 14, -4, 15], [-11, -18, 23, 6, -9, -20], [-33, -2, -3, 14, 7, 18], [-28, 0, -14, -8, -6, 12], [2, 30, 24, -1, -1, -16], [-24, 16, 16, -2, 8, -10], [22, 2, 8, -9, -4, 3], [27, 8, 23, 7, 5, 9], [-27, 24, 2, 14, -7, 7], [14, 18, 28, -11, -4, -5], [23, -4, -3, 20, -6, 9], [4, -10, 14, -2, -26, -10], [-29, -10, 17, 12, -12, -13], [18, 12, 8, 2, 6, -2], [-16, 12, -10, 10, 26, 16], [-17, 30, 8, -10, 8, 6], [42, 0, -4, 4, -10, 6], [34, 2, -12, -7, -2, 9], [-4, 8, 30, -12, -16, 4], [29, -4, -5, -20, -4, 5], [1, -4, 9, -3, -7, 11], [2, 8, 14, 20, -12, 6], [-3, -6, 11, -2, -1, -16], [-26, 4, -32, 14, 9, 19], [28, -10, 0, -5, -10, 13], [2, 4, 22, -2, -28, 0], [22, -10, 0, -4, -10, 6], [5, 2, -11, 10, 16, -11], [-14, -24, -16, -14, 6, -8], [26, -18, 4, 7, -24, 7], [-21, 6, 19, -12, -19, -2], [3, -22, -13, 24, -7, -2], [35, 10, 8, -2, 1, -15], [-13, -8, -1, -2, 11, 0], [18, -4, 10, 10, 4, 14], [17, 10, -1, 10, 18, -3], [44, 4, 20, -14, -2, -6], [-6, -16, -10, -8, -6, 0], [15, 4, 20, -8, 4, 0], [-15, 16, -3, 20, -15, 18], [14, 12, 18, 2, -2, 14], [25, -18, 16, 6, -3, -3], [-19, 20, 23, 5, -27, -7], [-3, 0, -14, -8, -2, -28], [10, -16, -6, 18, -10, 8], [28, 14, 16, -21, 3, -8], [20, 20, 36, -10, -4, 2], [-16, 16, 10, -14, -4, -14], [10, 22, 23, -2, -6, -7], [-20, -12, 10, 2, -18, -2], [-22, 8, -24, 12, -14, 8], [-9, 2, -5, 24, 5, -4], [-15, -6, -5, 36, 5, -8], [18, -6, -25, 8, 24, -23], [-12, 16, 12, 0, -4, -6], [-3, -2, -27, -8, 16, -5], [13, 14, -15, 10, 24, -1], [22, 10, 8, -40, 6, 2], [-9, 2, -3, 20, 0, 15], [-29, -8, -11, -22, 10, -21], [7, 12, -13, 6, 8, 19], [0, -12, -28, 6, -1, 11], [10, -12, 14, -6, -16, -22], [-44, 10, 2, -19, -7, 4], [-11, 2, -1, 12, -10, 15], [7, 6, 3, 10, -10, 1], [-24, 6, -6, -1, -5, 0], [4, 10, -18, -1, 21, -16], [15, -8, 1, 13, 15, 11], [13, 6, 5, 36, 14, -7], [-27, 12, 5, 13, 19, -5], [28, 20, 18, -26, -8, -24], [27, 8, 12, 12, 13, 7], [-4, -14, -14, -7, -13, 14], [37, 4, -9, -2, 12, -17], [-55, -10, 9, 4, -10, 13], [-32, 8, 4, -20, 20, -8], [26, -10, 20, -10, -4, -6]]
